diff --git a/lisp/progmodes/gud.el b/lisp/progmodes/gud.el
index 8518738d09e..245600a2f94 100644
--- a/lisp/progmodes/gud.el
+++ b/lisp/progmodes/gud.el
@@ -3871,6 +3871,42 @@ gud-lldb-stop
 
 (defun gud-lldb-marker-filter (string)
   "Deduce interesting stuff from process output STRING."
+  (let ((ofs 0))
+    (while (string-match (rx (* (not (in "\n\e")))
+                             (group "\e[")
+                             (? (group (+ digit)))
+                             (group (in "GJ")))
+                         string ofs)
+      (let ((bol (match-beginning 0))
+            (seq-start (match-beginning 1))
+            (arg (and (match-beginning 2)
+                      (string-to-number (match-string 2 string))))
+            (op-char (aref string (match-beginning 3)))
+            (seq-end (match-end 0)))
+        (cond ((eq op-char ?G)
+               ;; Move to absolute column. Argument is 1-based.
+               (let* ((goal-col (1- (or arg 1)))
+                      (current-col (current-column))
+                      (prefix-len (- seq-start bol))
+                      (eff-col (+ current-col prefix-len))
+                      (rel-move-left (- eff-col goal-col))
+                      (del-prefix (max 0 (min prefix-len rel-move-left)))
+                      (elide-start (- seq-start del-prefix))
+                      (elide-end seq-end)
+                      (del-chars (- rel-move-left del-prefix)))
+                 ;; FIXME: insert extra spaces if we move beyond eff-col
+                 (when (> del-chars 0)
+                   (delete-char (- del-chars)))
+                 (setq string
+                       (concat (substring string 0 elide-start)
+                               (substring string elide-end)))
+                 (setq ofs (+ bol (- prefix-len del-prefix)))))
+              ((eq op-char ?J)
+               ;; Erase in display
+               ;; FIXME
+               (setq string (concat (substring string 0 seq-start)
+                                    (substring string seq-end)))
+               (setq ofs seq-start))))))
   (cond
    ;; gud-info: (function-name args...)
    ((string-match (rx line-start (0+ blank) "gud-info:" (0+ blank)
